@@ -31,7 +31,7 @@ sub ModifyLimitsconf {
 
     if (&getGlobalConfig("ConfigureMiscPAM","limitsconf") eq "Y") {
 
-       &ActionLog("# sub ModifyLimitsconf\n");
+       &B_log("ACTION","# sub ModifyLimitsconf\n");
 
        unless (&GetDistro =~ /^OSX/) {
 	   my $file=&getGlobal('FILE', "limits.conf");
@@ -56,7 +56,7 @@ sub ModifyLimitsconf {
 
 #sub LimitRemoteRootLogins {
 
-   #&ActionLog("# sub LimitRemoteRootLogins\n");
+   #&B_log("ACTION","# sub LimitRemoteRootLogins\n");
 
 # Limit root to logging in only from console, closing up brute force 
 # remote attacks against root's password
@@ -76,7 +76,7 @@ sub LimitConsoleLogins {
 
     if ( ( &getGlobalConfig("ConfigureMiscPAM","consolelogin") eq "Y") and (&getGlobalConfig("ConfigureMiscPAM","consolelogin_accounts") )) {
 
-	&ActionLog("# sub LimitConsoleLogins\n");
+	&B_log("ACTION","# sub LimitConsoleLogins\n");
 
         # Optionally, limit console logins to root and the administrator.
 
@@ -90,12 +90,14 @@ sub LimitConsoleLogins {
 	# by making sure their PAM config files reference pam_access.so
 	my $pamFile;
 	foreach $pamFile ('login','xdm','gdm','kde') {
-		# 'login' is used by the getty apps (virtual consoles, serial)
-		# the others are X logins (standard, GNOME, KDE)
-		&B_append_line( &getGlobal('DIR', 'pamd') . "/${pamFile}",'^\s*[^#]*\s*account\s{1,}required\s{1,}/lib/security/pam_access.so\s*$', "account    required     /lib/security/pam_access.so\n");
+	    # 'login' is used by the getty apps (virtual consoles, serial)
+	    # the others are X logins (standard, GNOME, KDE)
+	    my $pamFQfilename = &getGlobal('DIR', 'pamd') . '/' . $pamFile;
+	    if ( -e $pamFQfilename ) {
+		&B_append_line( $pamFQfilename,'^\s*[^#]*\s*account\s{1,}required\s{1,}/lib/security/pam_access.so\s*$', "account    required     /lib/security/pam_access.so\n");
+	    }
 	}
-
-   }
+    }
 	    
 }
 
