@@ -1,6 +1,6 @@
 # Copyright 2001,2002 Hewlett-Packard Corporation
 # Licensed under the GNU General Public License
-# $Id: HP_UX.pm,v 1.56 2003/03/03 20:35:15 hpbuck Exp $
+# $Id: HP_UX.pm,v 1.57 2003/05/09 22:56:13 hpbuck Exp $
 
 package Bastille::HP_UX;
 
@@ -33,7 +33,7 @@ $ENV{BASH_ENV}="";
 sub StackNoExecute {
 
   if (&getGlobalConfig("HP_UX","stack_execute") eq "Y") {
-       &ActionLog("# sub Stack Execute \n");
+       &B_log("ACTION","# sub Stack Execute \n");
        # get binaries to be used...
        my $chmod = &getGlobal('BIN',"chmod"); 
        my $exportpath = &getGlobal('BIN',"umask")." 077; export PATH=/usr/bin;";
@@ -63,7 +63,7 @@ sub StackNoExecute {
 	       # going ahead and seting executable_stack param to 0
 	       if(&B_System("$exportpath $kmtune -s executable_stack=0","$exportpath $kmtune -s executable_stack=$kmexecq[3]\n")){
 		   #changed param successfully
-		   &ActionLog("Planned value of executable_stack tunable parameter is set to secure.\n");
+		   &B_log("ACTION","Planned value of executable_stack tunable parameter is set to secure.\n");
 
 		   if(&B_System("$exportpath $mk_kernel -o /stand/vmunix",
 			     "$exportpath $kmtune -s executable_stack=$kmexecq[3]; $mk_kernel -o /stand/vmunix\n")){
@@ -73,26 +73,26 @@ sub StackNoExecute {
                           # Just to make sure; this change should never be reverted
 		          &B_System("$chmod og-w " .$kernelbuilddir,"");
                        }
-		       &ActionLog("Kernel Build Successful: Reboot for changes to take affect\n");
+		       &B_log("ACTION","Kernel Build Successful: Reboot for changes to take affect\n");
 		       &B_TODO("\n\n---------------------------------\nStack Execute Permissions:\n" .
 			       "---------------------------------\n" .
 			       "Please reboot your system to enable stack execution protection.\n\n");
 		   }
 		   else{
-		       &ErrorLog("ERROR:   The kernel build failed. Could not compile kernel.\n");
+		       &B_log("ERROR","The kernel build failed. Could not compile kernel.\n");
 		   }
 	       }
 	       else{
-		   &ErrorLog("ERROR:   The kernel build failed. Could not set tunable parameter.\n");
+		   &B_log("ERROR","The kernel build failed. Could not set tunable parameter.\n");
 	       }
 	   }
 	   else{
-	       &ErrorLog("ERROR:   The kernel build failed.  Tunable parameters are in an unstable\n".
+	       &B_log("ERROR","The kernel build failed.  Tunable parameters are in an unstable\n".
                          "         state.  (Perhaps you forgot to reboot?)  Resolve the issue shown\n".
                          "         by the command $kmtune -d\n");
 	   }
          }else{
-	   &ActionLog("System already has kernel level stack execution protection. No changes made.\n");
+	   &B_log("ACTION","System already has kernel level stack execution protection. No changes made.\n");
          }
        }
    }
@@ -100,7 +100,7 @@ sub StackNoExecute {
 
 sub RestrictSwacls {
     if (&getGlobalConfig("HP_UX","restrict_swacls") eq "Y") {
-	&ActionLog("# sub Restrict SWACLS\n");
+	&B_log("ACTION","# sub Restrict SWACLS\n");
 	
 
 	my $swagentd = &getGlobal('BIN',"swagentd");
@@ -185,7 +185,7 @@ sub Ndd {
 
 
    if (&getGlobalConfig("HP_UX","ndd") eq "Y") {
-       &ActionLog("# sub Ndd\n");
+       &B_log("ACTION","# sub Ndd\n");
        # get all current values (-A) starting with any of the given names
        my $ch_rc = &getGlobal('BIN','ch_rc');
        my @origParams = 
@@ -276,7 +276,7 @@ sub Ndd {
 # about how the user should determine which ports are world listening
 sub Netstat {
     if (&getGlobalConfig("HP_UX","scan_ports") eq "Y") {
-	&ActionLog("# sub Netstat\n");
+	&B_log("ACTION","# sub Netstat\n");
 	my $netstat_text = 
 	    "Run a port scan to see what processes are still listening:\n" .
             "We recommend that you download the \"lsof\" tool from\n" .
@@ -366,7 +366,7 @@ sub OtherTools {
 
 sub MailConfig {
     if(&getGlobalConfig("HP_UX","mail_config") eq "Y"){
-	&ActionLog("# sub MailConfig");
+	&B_log("ACTION","# sub MailConfig");
         my $config_email='bastille-configs@fc.hp.com';
         my $todo_email='bastille-todo@fc.hp.com';
         my $feedback_email='bastille-feedback@fc.hp.com';
