@@ -85,7 +85,7 @@ sub do_Bastille {
 
     # Output answers to the script and display
 
-    &Output_Config_Files;
+    &outputConfig;
 
     # Run Bastille
 
@@ -319,6 +319,8 @@ sub Ask {
 	if ($return_index) {
 	    if ($answer) {
 		$Question{$index}{"answer"}=$answer;
+                $GLOBAL_CONFIG{$Question{$index}{'shortModule'}}{$index} = $answer;
+
 	    }
 	}
 	else {
