@@ -7,10 +7,9 @@ mkdir -p $RPM_BUILD_ROOT/usr/sbin
 mkdir -p $RPM_BUILD_ROOT/usr/lib/perl5/site_perl/Curses
 mkdir -p $RPM_BUILD_ROOT/usr/lib/Bastille
 mkdir -p $RPM_BUILD_ROOT/usr/share/Bastille
-mkdir -p $RPM_BUILD_ROOT/var/log/psad
-mkdir -p $RPM_BUILD_ROOT/var/lib/psad
-mkdir -p $RPM_BUILD_ROOT/var/run/psad
-mkdir -p $RPM_BUILD_ROOT/etc/psad
+mkdir -p $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+mkdir -p $RPM_BUILD_ROOT/usr/share/Bastille/Questions
+mkdir -p $RPM_BUILD_ROOT/usr/share/Bastille/FKL/configs/
 mkdir -p $RPM_BUILD_ROOT/var/lock/subsys/bastille
  
 cp AutomatedBastille $RPM_BUILD_ROOT/usr/sbin/
@@ -18,14 +17,22 @@ cp BastilleBackEnd $RPM_BUILD_ROOT/usr/sbin
 cp Bastille_Curses.pm $RPM_BUILD_ROOT/usr/lib/perl5/site_perl
 cp Bastille_Tk.pm $RPM_BUILD_ROOT/usr/lib/perl5/site_perl
 cp Curses/Widgets.pm $RPM_BUILD_ROOT/usr/lib/perl5/site_perl/Curses
-cp Default_Workstation $RPM_BUILD_ROOT/usr/share/Bastille
-cp Default_Workstation_plus_Firewall $RPM_BUILD_ROOT/usr/share/Bastille
 cp InteractiveBastille $RPM_BUILD_ROOT/usr/sbin
-cp Questions.txt $RPM_BUILD_ROOT/usr/share/Bastille
+# Questions.txt has been replaced by Modules.txt and Questions/
+#cp Questions.txt $RPM_BUILD_ROOT/usr/share/Bastille
+cp Modules.txt $RPM_BUILD_ROOT/usr/share/Bastille
+# New Weights file(s).
+cp Weights.txt $RPM_BUILD_ROOT/usr/share/Bastille
+# Castle graphic
+cp bastille.jpg $RPM_BUILD_ROOT/usr/share/Bastille/
+# Javascript file
+cp wz_tooltip.js $RPM_BUILD_ROOT/usr/share/Bastille/
 cp Credits $RPM_BUILD_ROOT/usr/share/Bastille
+cp FKL/configs/fkl_config_redhat.cfg $RPM_BUILD_ROOT/usr/share/Bastille/FKL/configs/
+
 cp RevertBastille $RPM_BUILD_ROOT/usr/sbin
 ln -s $RPM_BUILD_ROOT/usr/sbin/RevertBastille $RPM_BUILD_ROOT/usr/sbin/UndoBastille
-cp bastille $RPM_BUILD_ROOT/usr/sbin
+cp bin/bastille $RPM_BUILD_ROOT/usr/sbin
 chmod +x $RPM_BUILD_ROOT/usr/sbin/RevertBastille
 cp bastille-firewall $RPM_BUILD_ROOT/usr/share/Bastille
 cp bastille-firewall-reset $RPM_BUILD_ROOT/usr/share/Bastille
@@ -40,35 +47,9 @@ cp bastille-firewall-early.sh $RPM_BUILD_ROOT/usr/share/Bastille
 cp bastille-firewall-pre-audit.sh $RPM_BUILD_ROOT/usr/share/Bastille
 cp complete.xbm $RPM_BUILD_ROOT/usr/share/Bastille
 cp incomplete.xbm $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/psad $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/psadwatchd $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/kmsgsd $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/diskmond $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/psad-init $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/psad.8 $RPM_BUILD_ROOT/usr/man/man8
-mknod -m 600 $RPM_BUILD_ROOT/var/log/psadfifo p
-make -C psad/whois-4.5.29
-cp psad/whois-4.5.29/whois $RPM_BUILD_ROOT/usr/share/Bastille
-cp psad/psad_signatures $RPM_BUILD_ROOT/etc/psad
-cp psad/psad_auto_ips $RPM_BUILD_ROOT/etc/psad
-cp psad/psad.conf $RPM_BUILD_ROOT/etc/psad
-cp -r psad/Psad.pm $RPM_BUILD_ROOT/usr/share/Bastille
-PWDTMP=`pwd`
-cd $RPM_BUILD_ROOT/usr/share/Bastille/Psad.pm
-perl Makefile.PL
-make
-make test
-make install
-cd $PWDTMP
-cp -r psad/Unix-Syslog-0.98 $RPM_BUILD_ROOT/usr/share/Bastille
-PWDTMP=`pwd`
-cd $RPM_BUILD_ROOT/usr/share/Bastille/Unix-Syslog-0.98
-perl Makefile.PL
-make
-make test
-make install
-cd $PWDTMP
 cp ifup-local $RPM_BUILD_ROOT/usr/share/Bastille
+
+
 cp hosts.allow $RPM_BUILD_ROOT/usr/share/Bastille
 cp Bastille/AccountSecurity.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/Apache.pm $RPM_BUILD_ROOT/usr/lib/Bastille
@@ -81,6 +62,8 @@ cp Bastille/FilePermissions.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/FTP.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/Firewall.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/HP_API.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/OSX_API.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/LogAPI.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/HP_UX.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/IOLoader.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/Patches.pm $RPM_BUILD_ROOT/usr/lib/Bastille
@@ -101,6 +84,24 @@ cp Bastille/test_HP_UX.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/test_MiscellaneousDaemons.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/test_SecureInetd.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/test_Sendmail.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/test_BootSecurity.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/test_DisableUserTools.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/test_FilePermissions.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/test_Logging.pm $RPM_BUILD_ROOT/usr/lib/Bastille
+cp Bastille/test_Printing.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/TestAPI.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 cp Bastille/IPFilter.pm $RPM_BUILD_ROOT/usr/lib/Bastille
 
+
+
+cp OSMap/LINUX.bastille $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/LINUX.system $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/HP-UX.bastille $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/HP-UX.system $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/HP-UX.service $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/OSX.bastille $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+cp OSMap/OSX.system $RPM_BUILD_ROOT/usr/share/Bastille/OSMap
+
+for file in `cat Modules.txt` ; do
+   cp Questions/$file.txt $RPM_BUILD_ROOT/usr/share/Bastille/Questions
+done
